@@ -19,19 +19,36 @@
  * Author: Enrico Zini <enrico@enricozini.com>
  */
 
-#include <dballe/init.h>
-#include <dballe/core/verbose.h>
-#include <dballe/db/dba_db.h>
+#include "init.h"
+#include "config.h"
 
-dba_err dba_init()
+#ifdef HAVE_DBALLE_BUFREX
+void bufrex_codec_init(void);
+void bufrex_codec_shutdown(void);
+#endif
+#ifdef HAVE_DBALLE_MSG
+void aof_codec_init(void);
+void aof_codec_shutdown(void);
+#endif
+
+void dba_init()
 {
-	dba_verbose_init();
-	DBA_RUN_OR_RETURN(dba_db_init());
-
-	return dba_error_ok();
+#ifdef HAVE_DBALLE_BUFREX
+	bufrex_codec_init();
+#endif
+#ifdef HAVE_DBALLE_MSG
+	aof_codec_init();
+#endif
 }
 
 void dba_shutdown()
 {
-	dba_db_shutdown();
+#ifdef HAVE_DBALLE_BUFREX
+	bufrex_codec_shutdown();
+#endif
+#ifdef HAVE_DBALLE_MSG
+	aof_codec_shutdown();
+#endif
 }
+
+/* vim:set ts=4 sw=4: */
