@@ -1,7 +1,7 @@
 /*
  * DB-ALLe - Archive for punctual meteorological data
  *
- * Copyright (C) 2005,2006  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2008  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,37 +19,35 @@
  * Author: Enrico Zini <enrico@enricozini.com>
  */
 
-#ifndef DBA_INIT
-#define DBA_INIT
+#ifndef DBA_INIT_H
+#define DBA_INIT_H
 
 #ifdef  __cplusplus
 extern "C" {
 #endif
 
-/** @file
- * @ingroup dballe
- * Library initialisation and shutdown
+/** @init
+ * @ingroup core
+ * Initialise and shutdown the library
  */
 
-#include <dballe/err/dba_error.h>
-	
-/**
- * Initialize the library.
- *
- * This function needs to be called just once at the beginning of the work.
- */
-dba_err dba_init();
+#include <dballe/core/error.h>
 
 /**
- * Shutdown the library.
- *
- * This function needs to be called just once at the end of the work.
+ * Initialise DB-All.e
+ * @returns
+ *   The error indicator for the function.  See @ref error.h
+ */
+void dba_init();
+
+/**
+ * Shut down DB-All.e
  */
 void dba_shutdown();
 
-
 #ifdef  __cplusplus
 }
 #endif
 
+/* vim:set ts=4 sw=4: */
 #endif
