@@ -56,25 +56,27 @@
       call idba_seti(handle, "min", 4)
       call ensure_no_error("seti 7")
 
-      call idba_setc(handle, "leveltype", "1")
+      call idba_setc(handle, "leveltype1", "1")
       call ensure_no_error("setc 8")
       call idba_seti(handle, "l1", 1)
       call ensure_no_error("seti 9")
-      call idba_seti(handle, "l2", 1)
+      call idba_setc(handle, "leveltype2", "1")
       call ensure_no_error("seti 10")
+      call idba_seti(handle, "l2", 1)
+      call ensure_no_error("seti 11")
 
       call idba_seti(handle, "pindicator", 20)
-      call ensure_no_error("seti 11")
-      call idba_seti(handle, "p1", 1)
       call ensure_no_error("seti 12")
-      call idba_seti(handle, "p2", 1)
+      call idba_seti(handle, "p1", 1)
       call ensure_no_error("seti 13")
+      call idba_seti(handle, "p2", 1)
+      call ensure_no_error("seti 14")
       
       call idba_seti(handle, "rep_cod", 1)
-      call ensure_no_error("seti 14")
+      call ensure_no_error("seti 15")
 
       call idba_setc(handle, "B01011", "DB-All.e!")
-      call ensure_no_error("setc 15")
+      call ensure_no_error("setc 16")
 
 !     Perform the insert
       call idba_prendilo(handle)
@@ -137,11 +139,12 @@
          call ensure("enqdate i4", i4.eq.3)
          call ensure("enqdate i5", i5.eq.4)
          call ensure("enqdate i6", i6.eq.0)
-         call idba_enqlevel(handle, i1, i2, i3)
+         call idba_enqlevel(handle, i1, i2, i3, i4)
          call ensure_no_error("dammelo enqlevel")
-         call ensure("enqdate i1", i1.eq.1)
-         call ensure("enqdate i2", i2.eq.1)
-         call ensure("enqdate i3", i3.eq.1)
+         call ensure("enqlevel i1", i1.eq.1)
+         call ensure("enqlevel i2", i2.eq.1)
+         call ensure("enqlevel i3", i3.eq.1)
+         call ensure("enqlevel i4", i4.eq.1)
          call idba_enqtimerange(handle, i1, i2, i3)
          call ensure_no_error("dammelo enqtimerange")
          call ensure("enqdate i1", i1.eq.20)
@@ -215,72 +218,4 @@
     
       end program
 
-!!! ********************
-!!! * Utility functions
-!!! ********************
-
-!     Compute the length of a string
-      integer function istrlen(string)
-      character string*(*)
-      istrlen = len(string)
-      do while ((string(istrlen:istrlen).eq." " .or. &
-           string(istrlen:istrlen).eq."").and. &
-           istrlen.gt.0)
-         istrlen = istrlen - 1
-      enddo
-      return
-      end
-
-!     Continue execution only if there was no error
-      subroutine ensure_no_error(message)
-      character message*(*)
-      integer idba_error_code, ier
-      character buf*1000
-
-!      print *,"siamo a ",message
-      ier = idba_error_code()
-      if (ier.ne.0) then
-         print *,ier," in ",message
-         call idba_error_message(buf)
-         print *,buf(:istrlen(buf))
-         call idba_error_context(buf)
-         print *,buf(:istrlen(buf))
-         call idba_error_details(buf)
-         print *,buf(:istrlen(buf))
-         call exit (1)
-      end if
-      return
-
-      end
-
-!     Print an error if the given logical value is false
-      subroutine ensure(message, value)
-      character message*(*)
-      logical value
-
-      if (.not.value) then
-         print *,"Check failed in ",message
-         call exit (1)
-      end if
-      return
-
-      end
-
-      subroutine testcb(val)
-      integer val
-      character buf*1000
-
-      ier = idba_error_code()
-      if (ier.ne.0) then
-         print *,ier," testcb in ",val
-         call idba_error_message(buf)
-         print *,buf(:istrlen(buf))
-         call idba_error_context(buf)
-         print *,buf(:istrlen(buf))
-         call idba_error_details(buf)
-         print *,buf(:istrlen(buf))
-         call exit (1)
-      end if
-      return
-
-      end
+      include "check-utils.h"
